/* 
 *  Mezzanine - an overhead visual object tracker.
 *  Copyright (C) Andrew Howard 2002
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
/***************************************************************************
 * Desc: Track objects
 * Author: Andrew Howard
 * Date: 21 Apr 2002
 * CVS: $Id: track.c,v 1.2 2002/05/03 04:55:52 inspectorg Exp $
 ***************************************************************************/

#include <assert.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "smhal.h"
#include "smgeom.h"


// Track the ball
void track_update_ball(smipc_objectlist_t *objectlist);

// Track opposition robots
void track_update_opp(smipc_objectlist_t *objectlist);

// Track our own robots
void track_update_own(smipc_objectlist_t *objectlist);


// Track information
typedef struct
{
  smipc_objectlist_t *objectlist; // List of objects in the current frame
  smipc_ball_t *ball;             // List of ball properties
  smipc_opp_robot_t *opprobots;   // List of opp robot properties
  smipc_own_robot_t *ownrobots;   // List of own robot properties
} track_t;

// The one and only instance of the tracker
static track_t *track;


// Initialise the tracker
int track_init(smopt_t *opt, smipc_mmap_t *mmap)
{

  track = malloc(sizeof(track_t));
  memset(track, 0, sizeof(track_t));

  // Use the mmap'ed property list
  track->ball = &mmap->ball;
  track->opprobots = mmap->opp_robots;
  track->ownrobots = mmap->own_robots;
  
  return 0;
}


// Close the tracker
void track_term(smopt_t *opt)
{
  // Clean up
  free(track);
}


// Update tracks
void track_update(smipc_objectlist_t *objectlist)
{
  track_update_ball(objectlist);
  track_update_opp(objectlist);
  track_update_own(objectlist);
}


// Track the ball
// TODO: replace the DUMMY code below with something decent.
void track_update_ball(smipc_objectlist_t *objectlist)
{
  smipc_ball_t *ball;

  ball = track->ball;
  ball->pos_x = objectlist->ball.ox;
  ball->pos_y = objectlist->ball.oy;  
}


// Track opposition robots
// TODO: replace the DUMMY code below with something decent.
void track_update_opp(smipc_objectlist_t *objectlist)
{
  int i;
  smipc_object_t *object;
  smipc_opp_robot_t *robot;
  
  for (i = 0; i < SMIPC_MAX_OPPROBOTS; i++)
  {
    object = objectlist->opprobots + i;
    robot = track->opprobots + i;

    robot->onfield = object->valid;
    robot->pos_x = object->ox;
    robot->pos_y = object->oy;
  }
}


// Track our own robots
// TODO: replace the DUMMY code below with something decent.
void track_update_own(smipc_objectlist_t *objectlist)
{
  int i;
  smipc_object_t *object;
  smipc_own_robot_t *robot;
  
  for (i = 0; i < SMIPC_MAX_OWNROBOTS; i++)
  {
    object = objectlist->ownrobots + i;
    robot = track->ownrobots + i;

    robot->onfield = object->valid;
    robot->pos_x = object->ox;
    robot->pos_y = object->oy;
    robot->pos_a = object->oa;    
  }
}




