#include "Auxilliary.h"
#include <math.h>
#include "Config.h"
#include "Vector.h"

extern Config config;

Vector cellToCoord(int x, int y, double cellWidth, double cellHeight)
{
  Vector returnV;

  returnV.setX(x*cellWidth + cellWidth/3);
  returnV.setY(y*cellHeight + cellHeight/2);
  
  return returnV;
}

Vector coordToCell(double x, double y, double cellWidth, double cellHeight)
{
  Vector returnV;

  returnV.setX((int)(x/cellWidth + 0.5));
  returnV.setY((int)(y/cellHeight + 0.5));

  return returnV;
}


double normalize_angle(double a) {
	while (a >= M_PI) a -= 2*M_PI;
	while (a < -M_PI) a += 2*M_PI;
	return a;
}



double timeToMove(Vector startPos, Vector startOri, Vector endPos, Vector endOri) {
	Vector rayToPoint = endPos - startPos;
	double theta = rayToPoint.angle(); 
	double phi = startOri.angle();
	double delta = theta - phi;
	double distance = rayToPoint.magnitude();
	double alpha = rayToPoint.angle() - endOri.angle();
	alpha = normalize_angle(alpha);
	double falpha = fabs(alpha);
	double fdelta = fabs(delta);
	
	//return ( (.2107 * distance) + (43.5607 * falpha) + (43.4825 * fdelta) - (6.9824 * fdelta * fdelta) );
	double timeSteps = ( (.2107 * distance) + (43.5607 * falpha) + (43.4825 * fdelta) - (6.9824 * fdelta * fdelta) );
	return timeSteps / 100; //returns time in seconds, using approximation 100timeSteps = 1second
}

/*

double timeToMove(Vector startPos, Vector startOrient, Vector endPos, Vector endOrient) {

  //OLD ALGORITHM.  MOST LIKELY UNRELIABLE. REPLACE WITH REGRESSION RESULTS LATER

  float weightD = 1, weightA = 1, weightDelta = 1;
  
  Vector ray = endPos - startPos;
  double anglePhi = startOrient.angle();
  double c = config.robotClearDist;
  double d = ray.magnitude();

  double phistar = endOrient.angle();

  double theta = ray.angle();

  double alpha = theta - phistar;  //This is going to depend on which is larger... pretty messy
  
  double thetaP;

  if (fabs(alpha) < atan(c/d))
    thetaP = theta + alpha;
  else 
    thetaP = theta - sign(alpha)*atan(c/d);
  
  
  double delta = thetaP - anglePhi;

  double time = weightD * d + weightA * fabs(anglePhi-phistar) + weightDelta * fabs(delta);
  time /= (config.fieldDimensions.x());


  return time;
}
*/

double gauss(double radius, double sigma)
{
	//e^(-(x^2)/(2*sigma)^2); //Sigma influences width
	double exp;
	exp = (double)(-pow(radius, 2)/(pow(2 * sigma, 2)));
	return pow(M_E, exp);
}

int sign(double d) {
	if (d > 0) return 1;
	else return -1;
}
  
double min(double a, double b) {
	if (a <= b) return a;
	else return b;
}
