#include "CommServer.h"
#include "Config.h"

#include <iostream>

extern Config config;

CommServer::CommServer()
{
}

CommServer::CommServer(int port)
{
  init(port);
}

int CommServer::init(int port)
{
  int on = 1;  //socket option flag

  if ((sockfd = socket(AF_INET,SOCK_DGRAM,0)) == -1)
    {
      printf("Socket creation error");
      return -1;
    }
  
  sin.sin_family = AF_INET;

  if(config.unixPort)
    sin.sin_port = port;
  else
    sin.sin_port = htons(port);

  sin.sin_addr.s_addr = inet_addr("255.255.255.255");
  bzero(&(sin.sin_zero),8);
    
  if (setsockopt(sockfd, SOL_SOCKET, SO_BROADCAST, &on, sizeof(on)) < 0)
    {
      printf("socket change option error\n");
      return -1;
    }

  //if (setsockopt(sockf, SOL_SOCKET, SO_RCVBUF, 
  
  return 0;
}
 
int CommServer::sendData(Robot* robots)
{
  int numbytes = 0;
  char buffer[128];
  Vector motorCmds;

  //I'm pretty sure that Jacky's command server is set such that, if it receives %s, %d, %d, it defaults
  //to his measurements (inches, angles).  If you send %s, 0x%x, then you can use your own action set.

  int command;

  for(int i = 0; i < config.numOurRobots; i++)
    {
      motorCmds = robots[i].getMotors(); //returns vector (V_left, V_right), each ranging from -config.robotVelocity to config.robotVelocity
		double mtra, mtrc;
		mtra = motorCmds.x();
		mtrc = motorCmds.y();
		//normalize mtr commands to -7 -> 7
		mtra = (int)(((mtra)/config.robotVelocity) * 7 + 0.5);
		mtrc = (int)(((mtrc)/config.robotVelocity) * 7 + 0.5);
	

		//debug print
		if(config.verbose) 
		  printf("Robot %d, SENDING: mtra=%lf, mtrc=%lf\n", i, mtra, mtrc);
		
		//if mtrcommand is reverse, occupies range from 7->14
		if ((mtra < 0)) mtra = fabs(mtra) + 8;
		if ((mtrc < 0)) mtrc = fabs(mtrc) + 8;
		
      command = (((int)mtra << 4) & 0xf0) | (int)mtrc;
      //command = (((int)motorCmds.x() << 4) & 0xf0) | (int)motorCmds.y();

      //Our style
      sprintf(buffer, "%s,0x%x\n", (robots[i].getName()).c_str(), command);


      //Jacky style
      //      sprintf(buffer, "%s,%d,%d\n", (robots[i].getName()).c_str(), (int)motorCmds.x(), (int)motorCmds.y());


      if((numbytes = sendto(sockfd, buffer, strlen(buffer), 0, (struct sockaddr *)&sin, sizeof(struct sockaddr))) == -1)
	{
	  printf("Socket sending error:  Tried to send [%s]\n", buffer);
	  return -1;
	}
	else {
	  if(config.verbose)
	    printf("Sending [%s]\n", buffer);
	}
    }
  
  
  return 0;
}


/** THIS IS JUST A TEST FUNCTION **/ 
int CommServer::sendData()
{
  int numbytes = 0;
  char buffer[128];
  Vector motorCmds(1, 1);

  cout << "SENDING DATA " << endl;

  //sprintf(buffer, "blue1,%d,%d\n", (int)motorCmds.x(), (int)motorCmds.y());

  if((numbytes = sendto(sockfd, buffer, strlen(buffer), 0, (struct sockaddr *)&sin, sizeof(struct sockaddr))) == -1)
    {
      printf("Socket sending error:  Tried to send [%s]\n", buffer);
      return -1;
    }
    
  return 0;
}
 	 


CommServer::~CommServer()
{
  close(sockfd);
}
