#include "PotentialMap.h"
#include <iostream>

using namespace std;

PotentialMap::PotentialMap(Vector fieldDimensions, Vector numCells)
{
	field_width = fieldDimensions.x();
	field_height = fieldDimensions.y();
	num_cells_x = (int)numCells.x();
	num_cells_y = (int)numCells.y();
	cellWidth = field_width / ((double)num_cells_x);
	cellHeight = field_height / ((double)num_cells_y);

	int i;

	map = new double*[num_cells_x];

	for(i = 0; i < num_cells_x; i++)
	  {
	    map[i] = new double[num_cells_y];
	  }

	for (i = 0; i < num_cells_x; i++) {
		for (int j = 0; j < num_cells_y; j++) {
		  map[i][j] = 0;
		}
	}


}


PotentialMap::PotentialMap(const PotentialMap &rhs)
{
	field_width = rhs.field_width;
	field_height = rhs.field_height;
	num_cells_x = rhs.num_cells_x;
	num_cells_y = rhs.num_cells_y;

	cellWidth = field_width / ((double)num_cells_x);
	cellHeight = field_height / ((double)num_cells_y);

	int i;

	map = new double*[num_cells_x];

	for(i = 0; i < num_cells_x; i++)
	  {
	    map[i] = new double[num_cells_y];
	  }

	for(i = 0; i < num_cells_x; i++) {
		for(int j = 0; j < num_cells_y; j++) {
			map[i][j] = rhs.map[i][j];
		}
	}
}

PotentialMap& PotentialMap::operator=(const PotentialMap &rhs)
{

  if(&rhs == this)
    return *this;

  for(int i = 0; i < num_cells_x; i++)
    delete[] map[i];

  delete map;
  
  field_width = rhs.field_width;
  field_height = rhs.field_height;
  num_cells_x = rhs.num_cells_x;
  num_cells_y = rhs.num_cells_y;
  
  cellWidth = field_width / ((double)num_cells_x);
  cellHeight = field_height / ((double)num_cells_y);

  map = new double*[num_cells_x];
  
  for(int i = 0; i < num_cells_x; i++)
    {
      map[i] = new double[num_cells_y];
    }
  

  for(int i = 0; i < num_cells_x; i++) {
    for(int j = 0; j < num_cells_y; j++) {
      map[i][j] = rhs.map[i][j];
    }
  }

  return *this;
}


PotentialMap::~PotentialMap()
{
  for(int i = 0; i < num_cells_x; i++) {
    delete[] map[i];
  }

  delete map;
}

void PotentialMap::print() {

	for (int j = 0; j < num_cells_y; j++) {
	  for (int i = 0; i < num_cells_x; i++) {
	    cout << getCell(i, j) << " ";
	  }
	  cout << endl;
	}
}

void PotentialMap::clear() {

	for (int j = 0; j < num_cells_y; j++) {
	  for (int i = 0; i < num_cells_x; i++) {
	    map[i][j] = 0;
	  }
	}
}

void PotentialMap::setCell(int x, int y, double d)
{
  map[x][y] = d;
}

void PotentialMap::setCellReal(double x, double y, double d)
{
  Vector c = coordToCell(x, y);
  setCell((int)c.x(), (int)c.y(), d);
}

double PotentialMap::getCell(int x, int y)
{
  return map[x][y];
}

Vector PotentialMap::getMin()
{
  double min = getCell(0, 0);
  Vector minCell = Vector(0, 0);

  for(int i = 0; i < num_cells_x; i++)
    {
      for(int j = 0; j < num_cells_y; j++)
	{
	  if(getCell(i, j) < min)
	    {
	      min = getCell(i, j);
	      minCell.setX(i);
	      minCell.setY(j);
	    }
	}
    }

  return minCell;
}

Vector PotentialMap::getMax()
{
  double max = getCell(0, 0);
  Vector maxCell = Vector(0, 0);

  for(int i = 0; i < num_cells_x; i++)
    {
      for(int j = 0; j < num_cells_y; j++)
	{
	  if(getCell(i, j) > max)
	    {
	      max = getCell(i, j);
	      maxCell.setX(i);
	      maxCell.setY(j);
	    }
	}
    }

  return maxCell;
}
	    
PotentialMap PotentialMap::operator+(const PotentialMap &rhs)
{
	PotentialMap p = PotentialMap(Vector(field_width, field_height),
	Vector(num_cells_x, num_cells_y));

	for(int i = 0; i < num_cells_x; i++) {
		for(int j = 0; j < num_cells_y; j++) {
			//p.setCell(i,j,map[i][j] + rhs.getCell(i,j));
			p.setCell(i,j,map[i][j] + rhs.map[i][j]);
		}
	}

	return p;
}

Vector PotentialMap::cellToCoord(int x, int y)
{
  Vector returnV;

  //  cout << "CW: " << cellWidth << " CH:  " << cellHeight << "x: " << x << "y: " << y << endl;
  returnV.setX(x*cellWidth + cellWidth/2);

  //cout << returnV.x() << endl; 
  returnV.setY(y*cellHeight + cellHeight/2);

  //cout << returnV.y() << endl;
  
  return returnV;
}

Vector PotentialMap::coordToCell(double x, double y)
{
  Vector returnV;

  returnV.setX((int)(x/cellWidth + 0.5));
  returnV.setY((int)(y/cellHeight + 0.5));

  return returnV;
}

