#include "Robot.h"
#include "Auxilliary.h"
#include "Config.h"
#include "Brain.h"
#include <iostream>

extern Config config;

using namespace std;

Robot::Robot()
{
}

Robot::Robot(int id_)
{
  id = id_;
}

void Robot::init(Vector fieldDim, Vector numC)
{
  map = PotentialMap(fieldDim, numC);
}

Vector Robot::getPosition() { return position; }
double Robot::getOrientationDouble() { return orientation.angle(); }
Vector Robot::getOrientationVector() { return orientation; }
Vector Robot::getVelocity() { return velocity;}

//added by spt2002
void Robot::move( Vector endPos ){
	double clearDist = config.robotClearDist;
	double vel = 1.0;
	move( getPosition(), getOrientationVector(), endPos, getOrientationVector(), vel, clearDist );
}

//added by spt2002
void Robot::move( Vector endPos, Vector endOri ){
	double clearDist = config.robotClearDist;
	double vel = 1.0;
	move( getPosition(), getOrientationVector(), endPos, endOri, vel, clearDist );
}

void Robot::move( Vector startPos, Vector startOri, Vector endPos, Vector endOri, double vel ) {
	double clearDist = config.robotClearDist;
	move( startPos, startOri, endPos, endOri, vel, clearDist );
}
 
void Robot::move( Vector startPos, Vector startOri, Vector endPos, Vector endOri, double vel, double clearDist ) {
	Vector rayToPoint = endPos - startPos;

	double theta = rayToPoint.angle(); 
	double distance = rayToPoint.magnitude();
	double alpha = rayToPoint.angle() - endOri.angle();
	alpha = normalize_angle(alpha);
	double beta = atan2(clearDist,distance);
	double thetaMod;
	
	thetaMod = (sign(alpha)) * min(fabs(alpha), beta);
	
	/*Works
	if (fabs(alpha) < atan2(clearDist,distance)) 
		thetaMod = alpha;
	else thetaMod = sign(alpha) * atan2(clearDist,distance);
	*/

	double thetaPrime = theta + thetaMod;
	double phi = startOri.angle();
	double delta = thetaPrime - phi;
	double t = cos(delta)*cos(delta) * sign(cos(delta));
	double r = sin(delta)*sin(delta) * sign(sin(delta));

	//debug print
	/*
	printf("\n");
	printf("alpha, atan2(...): %f, %f\n", alpha, beta);
	printf("theta, theta', phi: %f, %f, %f\n", theta, thetaPrime, phi);
	printf("t, r: %f, %f\n", t, r);
	*/

	/*
	if(config.verbose)
	  printf("Robot %d performing action %s:\n", id, config.actionNames[action.actionID]);
	else
	  printf("  Robot%d: %s\t", id, config.actionNames[action.actionID]);

	if(config.verbose)
	  action.print();
	*/

	/*
	startPos.print("Robot Pos");
	startOri.print("Robot Ori");
	endPos.print("Desired End Pos");
	endOri.print("Desired End Ori");
	*/
	
	Vector v(vel*(t-r), vel*(t+r));

	
	//if turning hard, i.e. vel_r is opp sign of vel_l, slow down turn, else overcompensation
	if (v.x() / v.y() < 0) {
		v.normalize();
		v = v * (vel/3);
	}

	char velocityString[100];
	sprintf(velocityString, "   vel");
	v.print(velocityString);

	//return v;

	//set motors from -vel to vel
	setMotors(v.x(), v.y());
	
}


//maybe this should return a pointer; copying the map over is very expensive
//in the mean time, implementing getMapCell which gets a cell without requiring the whole map to be copied
PotentialMap Robot::getMap() { return map; }
double Robot::getMapCell(int x, int y) { return map.getCell(x,y); }

void Robot::setPosition(Vector pos) { position = pos; }
void Robot::setOrientation(double ori) { 
	orientation.setX(cos(ori));
	orientation.setY(sin(ori));
}
void Robot::setOrientation(Vector ori) { orientation = ori; }
void Robot::setVelocity(Vector vel) { velocity = vel; }
void Robot::setMap(PotentialMap map_) { map = map_; };
void Robot::setName(string name_) { name = name_; };

void Robot::setAction(Action a)
{
  action = a;
  if ( (a.actionID == SHOOT) &&
       ((a.position-position).magnitude() < config.robotShootClearDistRange ) )
    move(position, orientation, a.position, a.orientation, config.robotVelocity, config.robotShootClearDist);
  else  
    move(position, orientation, a.position, a.orientation, config.robotVelocity);
  //motors.print("Robot motors:");
}

