#include "VisionClient.h"

using namespace std;

extern Config config;

void *recvLoop(void *visionclient);

VisionClient::VisionClient()
{
  //recvThread = (pthread_t)NULL;
  msgReady = 0;
  threadAlive = 0;

}
  

int VisionClient::init(string host, int port)
{
  return connectVision(host, port);
}

VisionClient::~VisionClient()
{
  threadAlive = 0;
  close(visSocket);
}

int VisionClient::connectVision(string host, int port)
{
      struct hostent *h;

      if ((h = (struct hostent *)gethostbyname(host.c_str())) == NULL)
      {  
	return -1;
      }

      host = inet_ntoa(*((struct in_addr *)h->h_addr));
      visSocket = clientSocket(port);

      if(visSocket == -1)
	return -1;

      fflush( stdout );

      printf("Creating thread...\n");

      if((pthread_create(&recvThread, NULL, recvLoop, (void *)this)) != 0)
	{
	  perror("pthread");
	  exit(1);
	}
      
      
      return 0;
}

/*--------------------------------------------------------------------
  clientSocket()

  creates a socket and then attempts to make a connection to that 
  socket
  ------------------------------------------------------------------*/
int VisionClient::clientSocket (int port )
{

    int ss, optval = 1, bufSize = -1;
    struct sockaddr_in sin;

    char buffer[64000];

    bufSize = sizeof(buffer);
    
    //We're using datagrams
    if (( ss = socket( AF_INET, SOCK_DGRAM, 0 )) == -1) {
      printf("Fclient/socket\n");
      return -1;
    }

   
    sin.sin_family = AF_INET;
    sin.sin_port = htons(port);
    sin.sin_addr.s_addr =  INADDR_ANY;
    memset(&(sin.sin_zero), '\0', 8);

    /*

    if(setsockopt(ss, SOL_SOCKET, SO_RCVBUF, (size_t*)&bufSize, sizeof(bufSize)) < 0)
      {
	perror("setting SO_RCVBUF to 1");
	exit(1);
      }
    */
   
    if(-1 == bind(ss, (struct sockaddr *)&sin,sizeof(struct sockaddr) ))
      {
          perror("bind");
	  exit(1);
      }

    bufSize = 1;


       /* DEBUG */
    if(getsockopt(ss, SOL_SOCKET, SO_RCVBUF, (void *)&buffer, (socklen_t*)&bufSize) < 0)
      {
	perror("SO_RCVBUF");
	exit(1);
      }

    
  
    /* I'm not sure we need these, but I commented what I think they do below... (JAKE)
    /*
      //Determines who the "owner" of any SIGIOs is
    if(-1 == fcntl(ss, F_SETOWN, getpid()))  
          perror("fcntl F_SETOWN");

	  //Sends SIGIO when info is ready to be read or written
    if(-1 == fcntl(ss, F_SETFL, FASYNC))
          perror("fcntl F_SETFL, FASYNC");
    */

    /* return the socket descriptor */
    return( ss );

} /* end of client_sock() */

void VisionClient::getMessage(char* buffer)
{
  char buf[16284];
  memset(buf, 0, sizeof(buf));
  if(-1 == recvfrom( visSocket, buf, sizeof(buf), 0, 0, 0))
    perror("recvfrom");

  strcpy(buffer, buf);

}

void VisionClient::fillRobot(Robot* robot, string buf) 
{
  Vector position, velocity;
  float orientation;
  string nextVal;

  //Skip status values
  nextVal = advance(&buf, ' ');
  nextVal = advance(&buf, ' ');
  nextVal = advance(&buf, ' ');

  nextVal = advance(&buf, ' '); //Position X
  position.setX(atof(nextVal.c_str()));

  nextVal = advance(&buf, ' '); //Position Y
  position.setY(atof(nextVal.c_str()));

  nextVal = advance(&buf, ' '); //Skip height
  
  nextVal = advance(&buf, ' '); //Orientation
  orientation = atof(nextVal.c_str());

  nextVal = advance(&buf, ' '); //Velocity X
  velocity.setX(atof(nextVal.c_str()));

  nextVal = advance(&buf, ' '); //Velocity Y
  velocity.setY(atof(nextVal.c_str()));

  robot->setPosition(position);
  robot->setOrientation(orientation);
  robot->setVelocity(velocity);
}

void VisionClient::fillBall(Ball *ball, string buf)
{
  Vector position, velocity;
  string nextVal;

  //Skip status values
  nextVal = advance(&buf, ' ');
  nextVal = advance(&buf, ' ');
  nextVal = advance(&buf, ' ');

  nextVal = advance(&buf, ' '); //Position X
  position.setX(atof(nextVal.c_str()));

  nextVal = advance(&buf, ' '); //Position Y
  position.setY(atof(nextVal.c_str()));


  //Skip more status values
  nextVal = advance(&buf, ' ');
  nextVal = advance(&buf, ' ');

  nextVal = advance(&buf, ' '); //Velocity X
  velocity.setX(atof(nextVal.c_str()));

  nextVal = advance(&buf, ' '); //Velocity Y
  velocity.setY(atof(nextVal.c_str()));

  ball->setPosition(position);
  ball->setVelocity(velocity);
  
}

//Returns the portion of the string up to the sentinel, and leaves
//the rest in buf
string VisionClient::advance(string *buf, char sentinel)
{
  string line;
  int index;
  if((index = buf->find(sentinel)) != -1)
    {
      line = buf->substr(0, index);
      *buf = buf->substr(index+1, buf->length());
    }
      
  else
    {
      line = "";
      *buf = "";
    }

  return line;
      
}

void *recvLoop(void* arg)
{
  VisionClient *vc = (VisionClient *)arg;
  char buf[16384];


  vc->threadAlive = 1;
  
  while(vc->threadAlive)
    {
      //printf("In recvLoop\n");
      
      //shoddy mutex
      
      memset(buf, 0, sizeof(buf));
      if(-1 == recvfrom( vc->visSocket, buf, sizeof(buf), 0, 0, 0))
	perror("recvfrom");
      vc->buffer = string(buf);

      if(!vc->msgReady)
	{
	  vc->msgReady = 1;
	}

    }
}


int VisionClient::getData(Robot* robots, Ball* ball)
{
  string nextLine;

  //Block until we've finished reading the latest message
  while(!msgReady)
    {}

  //Skip first two lines
  nextLine = advance(&buffer, '\n');
  nextLine = advance(&buffer, '\n');

  while(nextLine != "")
    {
      nextLine = advance(&buffer, '\n');
      for(int i = 0; i < config.numRobots; i++)
	{
	  if(nextLine.find(ball->getName()) != -1)
	    fillBall(ball, nextLine);
	  else if(nextLine.find(robots[i].getName()) != -1)
	    fillRobot(&robots[i], nextLine);
	}

    }

  //We just read a message, so we set the flag to fill a new buffer for us
  msgReady = 0;

  return 0;
}
